<?php

namespace Pinpoint\RedirectManager\App\Router;

/**
 * Class NoRouteHandler
 * @package Pinpoint\RedirectManager\App\Router
 */
class NoRouteHandler implements \Magento\Framework\App\Router\NoRouteHandlerInterface
{
    private const ALL_STORE_VIEWS = "0";
    /**
     * @var \Pinpoint\RedirectManager\Model\NotFound
     */
    protected $notFound;

    /**
     * @var \Pinpoint\RedirectManager\Model\Redirect
     */
    protected $redirect;

    /**
     * @var array
     */
    protected $filters = ['app/etc', '/pub/media', '/media/', '/favicon.ico'];
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * NoRouteHandler constructor.
     * @param \Pinpoint\RedirectManager\Model\NotFound $notFound
     * @param \Pinpoint\RedirectManager\Model\Redirect $redirect
     */
    public function __construct(
        \Pinpoint\RedirectManager\Model\NotFound $notFound,
        \Pinpoint\RedirectManager\Model\Redirect $redirect,
        \Pinpoint\RedirectManager\Model\ResourceModel\Redirect\CollectionFactory $collectionFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        $this->notFound = $notFound;
        $this->redirect = $redirect;
        $this->storeManager = $storeManager;
        $this->collectionFactory = $collectionFactory;
    }

    /**
     * @param \Magento\Framework\App\RequestInterface $request
     * @return bool
     */
    public function process(\Magento\Framework\App\RequestInterface $request)
    {
        foreach ($this->filters as $filter) {
            if (strpos($request->getPathInfo(), $filter) !== false) {
                $request->setModuleName('cms')->setControllerName('noroute')->setActionName('index');
                return false;
            }
        }

        //log 404 to db table
        $this->notFound->log($request);

        //if we have a redirect for the URL, redirect to that page
        if ($redirect = $this->hasRedirect($request, (int) $this->storeManager->getStore()->getId())) {
            header(
                'Location: ' . $redirect->getData('target_path'),
                true,
                (int) $redirect->getData('redirect_type')
            );
            exit;
        }

        $request->setModuleName('cms')->setControllerName('noroute')->setActionName('index');
        return false;
    }

    /**
     * @param \Magento\Framework\App\RequestInterface $request
     * @param int $storeId
     * @return $this|bool
     */
    public function hasRedirect(\Magento\Framework\App\RequestInterface $request, int $storeId)
    {
        $collection = $this->collectionFactory->create()
            ->addFieldToFilter('requested_path', $request->getPathInfo())
            ->addFieldToFilter('store_id', ['in' => [$storeId, self::ALL_STORE_VIEWS]]);
        
        if ($collection->getSize() === 0) {
            $collection = $this->collectionFactory->create()
                ->addFieldToFilter('requested_path', rtrim($request->getPathInfo(), '/'))
                ->addFieldToFilter('store_id', ['in' => [$storeId, self::ALL_STORE_VIEWS]]);
        }

        if ($collection->getSize()) {
            return $collection->getFirstItem();
        }
        
        return false;
    }

}